
option add *highlightThickness 1
option add *highlightColor black
option add *highlightBackground black
option add *activeBackground #b0a090
option add *background #b0b0b0
option add *foreground black
option add *borderWidth 1
option add *relief raised
option add *font -*-lucidatypewriter-medium-*-*-*-12-*-*-*-*-*-*-*

option add *Frame.background #909090
option add *Frame.borderWidth 1
option add *Frame.highlightThickness 0

option add *Button.padX 1
option add *Button.padY 1

option add *Label.padX 1
option add *Label.padY 1

option add *Menu.padX 1
option add *Menu.padY 1
option add *Menu.borderWidth 1

option add *Menubutton.padX 1
option add *Menubutton.padY 2

option add *ThemeLabel.foreground white
option add *ThemeLabel.font "Times 12 bold"

proc theme:loadXPM {win f} {
	set xpmIn [open $f r]
	fconfigure $xpmIn -translation binary
	set xpmData [read $xpmIn]
	close $xpmIn

	$win $xpmData
}

proc theme:loadNew {themeWin win} {
	set sel [$win curselection]
	set f [$win get $sel]
	theme:loadXPM $themeWin $f
}

proc about {} {
	frame .about
	pack [label .about.title -text About -bd 0] -fill x
	pack [message .about.msg -text {Tk_Theme is free software by George Peter Staplin} -aspect 300] -fill both
	pack [button .about.done -text Done -command {destroy .about}]
	update idletasks
	
	set x [expr {([winfo width .] / 2) - ([winfo reqwidth .about] / 2)}]
	set y [expr {([winfo height .] / 2) - ([winfo reqheight .about] / 2)}]
	
	place .about -x $x -y $y
}

proc winPuts {win} {
	if {![winfo exists $win]} {
		return
	}
	puts [format %ld [winfo id $win]]
}

proc theme:frame:addUp {win type} {
	set p [winfo $type $win]
	set aWin [winfo parent $win]
	while {[winfo toplevel $aWin] != $aWin} {
		set p [expr {$p + [winfo $type $aWin]}]
		set aWin [winfo parent $aWin]
	}
	return $p
}

proc theme:frame:findMax {winList type} {
	set max 1
	foreach win $winList {
		set rWidth [winfo $type $win]
		if {$rWidth > $max} {
			set max $rWidth
		}
	}
	return $max
}


set ::hello ABC
set ::def(k) 123
proc main {} {
	source ./theme:menu.tcl

	wm title . "Tk Theme Demo"
	tk appname "Tk Theme Demo"
	tk_focusFollowsMouse

	pack [theme:frame .f] -fill both -expand 1 -side top
	theme:loadXPM .f circle_weave.xpm

	frame .f.menubar
	pack [theme:menu .f.menubar.file . File] -side left
	
	set fm {.f.menubar.file}
	
	$fm add button -text {Set variable ::hello to ABC 456} -command {set ::hello "ABC 456"}
	$fm add button -text {Set variable ::def(k) to DEF 123} -command {set ::def(k) "DEF 123"}
	$fm add button -text Exit -command exit

	
	pack [theme:menu .f.menubar.about . About] -side left
	set am {.f.menubar.about}
	
	$am add button -text About -command about

	#theme:frame:translucent $newWin black
	
	theme:frame .f.rel 
	#pack [theme:label .f.rel.l -text "Relativity:"] -side left
	pack [theme:label .f.rel.l -textvariable ::hello] -side left
	pack [theme:label .f.rel.l2 -textvariable ::def(k)] -side left
	
	#.f.rel.l "Relativity: "
	pack [scale .f.rel.s -orient horizontal] -side left
	.f.rel.s set 50
	
	theme:frame .f.con
	pack [theme:frame .f.con.fl] -side left
	pack [theme:label .f.con.fl.myconf] -side left
	.f.con.fl.myconf "My Confidence: "
	pack [scale .f.con.fl.s -orient horizontal] -side left
	.f.con.fl.s set 65
	
	pack [theme:frame .f.con.fr] -side right
	pack [theme:label .f.con.fr.yourconf] -side left
	.f.con.fr.yourconf "Your Confidence: "
	pack [scale .f.con.fr.s -orient horizontal] -side left
	.f.con.fr.s set 85
	
	theme:frame .f.h
	pack [theme:label .f.h.l] -side left
	.f.h.l "Warning: "
	pack [button .f.h.b -text "Drop Bomb!" -command {puts OUCH; set ::hello Kaboom; set ::def(k) "Pardon Me"}] -side left

	theme:frame .f.t
	pack [theme:label .f.t.l]
	.f.t.l "Type Here:"
	pack [text .f.t.t -width 12 -height 3 -fg green -bg royalblue]
	pack [theme:label .f.t.l2]
	.f.t.l2 "And Here:"
	pack [entry .f.t.e -width 12]
	
	theme:label .f.uni
	#A spice of Unicode
	.f.uni "Unicode in a theme:label: \u0100 \u0101 \u0102 \u0103 \u0104"

	theme:frame .f.fsl
	pack [theme:label .f.fsl.l]
	.f.fsl.l "Select a Theme:"
	
	pack [listbox .f.fsl.list]

	foreach f [lsort -dictionary [glob *.xpm]] {
		.f.fsl.list insert end $f
	}

	bind .f.fsl.list <ButtonRelease-1> "theme:loadNew .f %W"

	pack .f.menubar -side top -anchor w
	pack .f.rel -side top -anchor w
	
	pack .f.h -side top -padx 5 -pady 5 -anchor w

	pack .f.con -side top -padx 20 -pady 20 -anchor c
	
	pack .f.uni
	pack .f.t
	pack .f.fsl -side bottom -pady 10

	set fi [open cursor.xpm r]
	#set fi [open 3D_Cursor-4_mod.xpm r]
	fconfigure $fi -translation binary
	set theData [read $fi]
	close $fi


	theme:cursor . cursor $theData

	pack [button .dest -text Destroy -command {destroy .f.h}]
}
main
