
proc theme:menu:addUp {win type relativeTo} {
	set res 0
	while {![string equal $win $relativeTo]} {
		set res [expr {$res + [winfo $type $win]}]
		set win [winfo parent $win]
	}
	return $res
}

proc theme:menu:display {win} {
	upvar #0 _themeMenu$win ar
	
	if {![winfo exists $ar(menuFrame)]} {
		return
	}
	
	if {[winfo ismapped $ar(menuFrame)]} {
		place forget $ar(menuFrame)
		return
	}

	set x [theme:menu:addUp $win.menuButton x $ar(master)]
	set y [theme:menu:addUp $win.menuButton y $ar(master)]
	set y [expr {$y + [winfo height $win]}]
	place $ar(menuFrame) -x $x -y $y
	raise $ar(menuFrame)
}

proc theme:menu:instanceCmd {win cmd args} {
	upvar #0 _themeMenu$win ar
	
	if {$cmd == "add"} {
		set type [lindex $args 0]
		set i 0
		set menuFrame $ar(menuFrame)
		
		while {[winfo exists $menuFrame.$i]} {
			incr i
		}
		
		set wargs [lrange $args 1 end]
		
		set cidx [lsearch $wargs "-command"]
		
		if {$cidx != -1} {
			incr cidx
			set cmd [lindex $wargs $cidx]
			append cmd "; theme:menu:display $win"
			set wargs [lreplace $wargs $cidx $cidx $cmd]
		}
		if {![catch {$type .__tmp -anchor w}]} {
			append wargs " -anchor w"
		}
		destroy .__tmp

		pack [eval $type $menuFrame.$i $wargs] -fill x
		return $menuFrame.$i
	} elseif {$cmd == "clear"} {
		foreach child [winfo children $ar(menuFrame)] {
			destroy $child
		}
		$ar(menuFrame) config -width 1 -height 1
	} elseif {$cmd == "title"} {
		set newTitle [lindex $args 0]
		$win.menuButton config -text $newTitle
	} else {
		return -code error "invalid subcommand: $cmd"
	}
}

proc theme:menu {win master txt} {
	frame $win -highlightthickness 0 -bd 0 -class ThemeMenu
	
	pack [button $win.menuButton -text $txt -command [list theme:menu:display $win]]

	upvar #0 _themeMenu$win ar
	
	set ar(master) $master
	set tmpName [string map {"." "_"} $win]
	
	set ar(menuFrame) [frame $master.$tmpName]

	rename $win theme:menu:orig$win
	
	interp alias {} $win {} theme:menu:instanceCmd $win

	return $win
}

return;
#TEST CODE
proc main {} {
	pack [frame .toolbar] -fill x
	pack [theme:menu .toolbar.m . {Add Mode}] -side left
	pack [theme:menu .toolbar.m2 . {Hello World}] -side left
	pack [text .t -height 20 -width 50] -fill both -side bottom
	
	interp alias {} tout {} .t insert end
	
	set m1 {.toolbar.m}
	set m2 {.toolbar.m2}
	$m1 add button -text Hello -command {tout "Hello\n"}
	$m1 add button -text What -command {tout "What?\n"}
	$m1 add button -text {I'm a button} -command {tout "You pressed me!\n"}
	$m1 add button -text {Change Title} -command [list $m1 title "Boing!"]
	$m1 add button -text Clear -command [list $m1 clear]
	$m1 add checkbutton -text {Check Me} 
	$m1 add label -text "I'm a label."
	set e [$m1 add entry]
	$e insert end "I'm an entry widget."
	
	$m2 add button -text {:)} -command {tout "Happy\n"}
	$m2 add button -text {:(} -command {tout "Sad\n"}
	$m2 add button -text {:/} -command {tout "Grumble\n"}
	$m2 add button -text {Reset} -command {foreach child [winfo children .] {destroy $child}; main}
}
main
