#include "theme.h"

typedef struct ThemeCursor {
	Display *dis;
	Cursor cursor;
	Pixmap pix;
	Window xWin;
	Window xWinRoot;
	Window xParent;
	unsigned int nWidth;
	unsigned int nHeight;
	int nScreen;
	int bMapped;
	GC cursorGC;
	int nDepth;
} ThemeCursor;

static char dummyCursorData[] = {
	"#define d_width 1\n"
	"#define d_height 1\n"
	"static unsigned char d_bits[] = {0x01};\n"
};

static void ThemeCursorDraw (ClientData clientData) {
	ThemeCursor *wPtr = (ThemeCursor *) clientData;
	Pixmap tmpPix;

	XRaiseWindow (wPtr->dis, wPtr->xWin);
	
	tmpPix = XCreatePixmap (wPtr->dis, wPtr->xWin, wPtr->nWidth, wPtr->nHeight, wPtr->nDepth);
	
	XCopyArea (wPtr->dis, 
		wPtr->pix, tmpPix, 
		wPtr->cursorGC,
		0, 0, 
		wPtr->nWidth, wPtr->nHeight,
		0, 0
	);
	XCopyArea (wPtr->dis, 
		tmpPix, wPtr->xWin, 
		wPtr->cursorGC,
		0, 0, 
		wPtr->nWidth, wPtr->nHeight,
		0, 0
	);
	XFreePixmap (wPtr->dis, tmpPix);
}

static void ThemeCursorMove (ThemeCursor *wPtr, int x, int y) {
	if (wPtr->bMapped == 0) {
		XMapWindow (wPtr->dis, wPtr->xWin);
		wPtr->bMapped = 1;
	}
	
	XRaiseWindow (wPtr->dis, wPtr->xWin);
	XMoveWindow (wPtr->dis, wPtr->xWin, (x + 1), (y + 1));
}

static int ThemeCursorInstanceCmd OBJ_CMD_ARGS {
	ThemeCursor *wPtr = (ThemeCursor *) clientData;
	int i;
	char *arg;
	int len = 0;

	for (i = 0; i < objc; ++i) {
		arg = Tcl_GetStringFromObj (objv[i], &len);
		if (strncmp ("move", arg, len) == 0) {
			int x;
			int y;
			
			if ((i + 2) >= objc) {
				Tcl_WrongNumArgs (interp, 2, objv, "x y");
				return TCL_ERROR;
			}
			
			++i;
			if (Tcl_GetIntFromObj (interp, objv[i], &x) != TCL_OK) {
				return TCL_ERROR;
			}
			++i;
			if (Tcl_GetIntFromObj (interp, objv[i], &y) != TCL_OK) {
				return TCL_ERROR;
			}
		
			/*fprintf (stderr, "x %d y %d\n", x, y);*/
			ThemeCursorMove (wPtr, x, y);
		
		} else if (strncmp ("unmap", arg, len) == 0) {
			wPtr->bMapped = 0;
			XUnmapWindow (wPtr->dis, wPtr->xWin);
		} 
	}

	/*Possibly add the ability to change the cursor data.*/

	return TCL_OK;
}

static int GetPixmapSize (Display *dis, Pixmap p, int *width, int *height) {
	Window root;
	int x, y;
	int bd;
	int depth;
	
	if (XGetGeometry (dis, p, &root, &x, &y, width, height, &bd, &depth)) {
		return 0;
	}
	return 1;
}

static int GenericHandler (ClientData clientData, XEvent *eventPtr) {
	ThemeCursor *wPtr;
	if (clientData == NULL) {
		return 0;
	}

	wPtr = (ThemeCursor *) clientData;

	if (eventPtr->type == Expose && eventPtr->xexpose.window == wPtr->xWin) {
		Tcl_DoWhenIdle (ThemeCursorDraw, clientData);
		return 1;
	} else if (eventPtr->type == ConfigureNotify && eventPtr->xconfigure.window == wPtr->xWin) {
		Tcl_DoWhenIdle (ThemeCursorDraw, clientData);
		return 1;
	} else if (eventPtr->type == MotionNotify) {
		int x = 1;
		int y = 1;
		Window root;
		Window child;
		int rootx;
		int rooty;
		unsigned int pmask;
		
		XQueryPointer (wPtr->dis, wPtr->xParent, &root, &child, &rootx, &rooty, &x, &y, &pmask);
		
		XDefineCursor (wPtr->dis, eventPtr->xmotion.window, wPtr->cursor);
		ThemeCursorMove (wPtr, x, y);
	}
	
	return 0;
}


int ThemeCursorCmd OBJ_CMD_ARGS {
	char *path;
	char *cmd;
	char *xpmData;
	
	XSetWindowAttributes xattr;
	ThemeCursor *wPtr = NULL;
	Pixmap pixShape;
	int eventBase;
	int errorBase;
	Tk_Window tkmain;
	Window xWin;
	Display *dis;
	int nScreen = 0;
	Window xWinRoot;
	Cursor cursor;
	Pixmap cursorPixmap;
	Pixel blackPixel;
	int nDepth = 0;
	XColor blackColor;
	Colormap cmap;
	Window xParent;
	
	/*theme:cursor .parent instanceCmd xpmData*/
	if (objc != 4) {
		Tcl_WrongNumArgs (interp, 1, objv, ".parent instanceCmd xpmData");
		return TCL_ERROR;
	}
	
	path = Tcl_GetString (objv[1]);
	cmd = Tcl_GetString (objv[2]);
	xpmData = Tcl_GetString (objv[3]);

	tkmain = Tk_MainWindow (interp);
	dis = Tk_Display (tkmain);
	nScreen = Tk_ScreenNumber (tkmain);
	xWinRoot = DefaultRootWindow (dis);
	nDepth = Tk_Depth (tkmain);
	blackPixel = BlackPixel (dis, nScreen);

	cmap = DefaultColormap (dis, nScreen);

	{
		Tk_Window tkwinTmp;
	
		tkwinTmp = Tk_NameToWindow (interp, path, tkmain);
	
		if (tkwinTmp == NULL) {
			return TCL_ERROR;
		}
		
		Tk_MakeWindowExist (tkwinTmp);
		xParent = Tk_WindowId (tkwinTmp);
	}
	
	if (!XShapeQueryExtension (dis, &eventBase, &errorBase)) {
		Tcl_SetResult (interp, "Your display doesn't appear to support the X shape extension.", TCL_STATIC);
		return TCL_ERROR;
	}
	
	cursorPixmap = XCreateBitmapFromData (dis, xWinRoot, dummyCursorData, 1, 1);
	
	if (!XParseColor (dis, cmap, "black", &blackColor)) {
		Tcl_SetResult (interp, "error parsing black color", TCL_STATIC);
		return TCL_ERROR;
	}

	if (!XAllocColor (dis, cmap, &blackColor)) {
		Tcl_SetResult (interp, "error allocating color", TCL_STATIC);
		return TCL_ERROR;
	}

	cursor = XCreatePixmapCursor (dis, cursorPixmap, cursorPixmap, &blackColor, &blackColor, 0, 0);
		
	xattr.override_redirect = 1;
	xattr.cursor = cursor;
	xattr.backing_store = 1;
	xattr.event_mask = ExposureMask | StructureNotifyMask | FocusChangeMask;

	xWin = XCreateWindow (dis, xParent, 
		1, 1,
		1, 1,
		0,
		nDepth,
		InputOutput,
		CopyFromParent,
		CWOverrideRedirect | CWCursor | CWBackingStore | CWEventMask, &xattr
	);
	
	wPtr = (ThemeCursor *) ckalloc (sizeof (ThemeCursor));
	
	wPtr->dis = dis;
	wPtr->xWin = xWin;
	wPtr->cursor = cursor;
	wPtr->pix = None;
	wPtr->cursorGC = XCreateGC (wPtr->dis, wPtr->xWin, 0, 0);
	wPtr->nScreen = nScreen;
	wPtr->xWinRoot = xWinRoot;
	wPtr->nWidth = 1;
	wPtr->nHeight = 1;
	wPtr->bMapped = 0;
	wPtr->nDepth = nDepth;
	wPtr->xParent = xParent;

	if (XpmCreatePixmapFromBuffer (wPtr->dis, wPtr->xWin, xpmData, &wPtr->pix, &pixShape, NULL)) {
		Tcl_SetResult (interp, "bad xpm", TCL_STATIC);
		ckfree ((char *)wPtr);
		return TCL_ERROR;
	}
	
	if (wPtr->pix == None) {
		Tcl_SetResult (interp, "invalid xpm pixmap", TCL_STATIC);
		ckfree ((char *)wPtr);
		return TCL_ERROR;
	}
	
	XShapeCombineMask (wPtr->dis, wPtr->xWin, ShapeBounding, 0, 0, pixShape, ShapeSet);
	
	GetPixmapSize (wPtr->dis, wPtr->pix, &wPtr->nWidth, &wPtr->nHeight);

	XResizeWindow (wPtr->dis, wPtr->xWin, wPtr->nWidth, wPtr->nHeight);
	Tk_CreateGenericHandler (GenericHandler, wPtr);
	Tcl_CreateObjCommand (interp, cmd, ThemeCursorInstanceCmd, (ClientData) wPtr, (Tcl_CmdDeleteProc *) NULL);
	XGrabPointer (wPtr->dis, 
		wPtr->xWin, 1, 
		PointerMotionMask | PointerMotionHintMask, 
		GrabModeAsync, GrabModeAsync,
		wPtr->xWin,
		cursor,
		CurrentTime
	);
	
	return TCL_OK;
}
