
#include "theme.h"

typedef struct ThemeLabel {
	Tk_Window tkwin;
	Window xWin;
	Display *dis;
	char *text;
	char *path;
	int depth;
	Colormap colormap;
	int textPixWidth;
	int textPixHeight;
	int textPixY;
	Tk_Font tkfont;
	GC textGC;
	int numChars;
	int screen;
	XColor fgCol;
	int noBGPixmap;
	char *varName;
} ThemeLabel;

static void ThemeLabelEventDestroy (ThemeLabel *wPtr) {
	if (wPtr->tkfont != NULL) {
		XFreeColors (wPtr->dis, wPtr->colormap, &wPtr->fgCol.pixel, 1, 0L);
		XFreeColormap (wPtr->dis, wPtr->colormap);
		Tk_FreeFont (wPtr->tkfont);
		free (wPtr->text);

		if (wPtr->varName != NULL) {
			free (wPtr->varName);
		}
	}

	XFreeGC (wPtr->dis, wPtr->textGC);
	free (wPtr->path);
	ckfree ((char *)wPtr);
}

/*TODO draw into a pixmap and then XCopyArea*/
static void ThemeLabelDraw (ThemeLabel *wPtr) {
	
	
	if (wPtr->noBGPixmap == 0) {
		Tk_SetWindowBackgroundPixmap (wPtr->tkwin, ParentRelative); 
	}

	if (wPtr->tkfont == NULL) {
		return;
	}

#define DOUBLE_BUFFER
#ifdef DOUBLE_BUFFER
	{
		int wWidth = Tk_Width (wPtr->tkwin);
		int wHeight = Tk_Height (wPtr->tkwin);
		XGCValues xgcval;
		GC clearGC;
		GC setGC;
		GC copyGC;
		XGCValues textVal;
		
		Pixmap bufferPix = XCreatePixmap (wPtr->dis, wPtr->xWin, wWidth, wHeight, wPtr->depth);
		Pixmap clipMaskPix = XCreatePixmap (wPtr->dis, wPtr->xWin, wWidth, wHeight, 1);


		xgcval.graphics_exposures = 0;
		xgcval.function = GXclear;
		clearGC = XCreateGC (wPtr->dis, clipMaskPix, GCFunction | GCGraphicsExposures, &xgcval);

		XFillRectangle (wPtr->dis, clipMaskPix, clearGC, 0, 0, wWidth, wHeight);
		/*memset (&xgcval, 0, sizeof (XGCValues));*/
		
		xgcval.function = GXset;
		setGC = XCreateGC (wPtr->dis, clipMaskPix, GCFunction | GCGraphicsExposures, &xgcval);
		/*It seems that I can't copy between the GC's with XCopyGC because they are too different; 1 vs >1 depth.*/
		/*I can however get the values and change the setGC.*/
		
		XGetGCValues (wPtr->dis, wPtr->textGC, GCFont | GCForeground, &textVal);
		xgcval.font = textVal.font;
		XChangeGC (wPtr->dis, setGC, GCFont, &xgcval);
		
		Tk_DrawChars (wPtr->dis, clipMaskPix, 
			setGC, wPtr->tkfont,
			wPtr->text, strlen (wPtr->text),
			1, (wPtr->textPixY + 1)
		);
		xgcval.graphics_exposures = 0;
		xgcval.function = GXcopy;
		xgcval.clip_mask = clipMaskPix;
		xgcval.font = textVal.font;
		xgcval.foreground = textVal.foreground;
		copyGC = XCreateGC (wPtr->dis, wPtr->xWin, GCFunction | GCClipMask | GCFont | GCForeground | GCGraphicsExposures, &xgcval);

		Tk_DrawChars (wPtr->dis, bufferPix,
			wPtr->textGC, wPtr->tkfont,
			wPtr->text, strlen (wPtr->text),
			1, (wPtr->textPixY + 1)
		);
		
		XCopyArea (wPtr->dis, bufferPix, wPtr->xWin, copyGC, 0, 0, wWidth, wHeight, 0, 0);
		
		XFreeGC (wPtr->dis, clearGC);
		XFreeGC (wPtr->dis, setGC);
		XFreeGC (wPtr->dis, copyGC);
		XFreePixmap (wPtr->dis, bufferPix);
		XFreePixmap (wPtr->dis, clipMaskPix);
	}
return;
#endif

	Tk_DrawChars (wPtr->dis,
		wPtr->xWin, 
		wPtr->textGC, wPtr->tkfont,
		wPtr->text, strlen (wPtr->text),
		1, (wPtr->textPixY + 1)
	);
}


static void ThemeLabelEvent (ClientData clientData, XEvent *eventPtr) {
	ThemeLabel *wPtr = (ThemeLabel *) clientData;
	
	if (eventPtr->type == DestroyNotify) {
		ThemeLabelEventDestroy (wPtr);
		return;
	}

	if (eventPtr->type == ConfigureNotify) {
		ThemeSendExposeToSelfAndChildren (wPtr->dis, wPtr->xWin);
		return;
	}

	ThemeLabelDraw (wPtr);
}

static int ThemeLabelInstanceCmd OBJ_CMD_ARGS;

static char *ThemeLabelTraceWrite (ClientData clientData, Tcl_Interp *interp, char *part1, char *part2, int flags) {
	ThemeLabel *wPtr = (ThemeLabel *) clientData;
	Tcl_Obj *arObj[2];
	char *str;
	
	/*
	fprintf (stderr, "ThemeLabelTraceWrite part1 %s part2 %s\n", part1, part2);
	*/

	if (wPtr->varName == NULL) {
		return NULL;
	}

	str = Tcl_GetVar2 (interp, wPtr->varName, NULL, TCL_GLOBAL_ONLY);
	if (str == NULL) {
		/*The variable isn't set yet or doesn't exist.*/
		str = "";
	}
	
	arObj[0] = Tcl_NewStringObj (wPtr->path, strlen (wPtr->path));
	arObj[1] = Tcl_NewStringObj (str, strlen (str));
	
	ThemeLabelInstanceCmd ((ClientData) wPtr, interp, 2, arObj);
	
	Tcl_DecrRefCount (arObj[0]);
	Tcl_DecrRefCount (arObj[1]);
	
	ThemeSendExposeToSelfAndChildren (wPtr->dis, wPtr->xWin);
	ThemeLabelDraw (wPtr);

	return NULL;
}

static int ThemeLabelInstanceCmd OBJ_CMD_ARGS {
	ThemeLabel *wPtr = (ThemeLabel *) clientData;
	char *text = NULL;
	int length;
	Tk_FontMetrics fm;
	char *fontName = NULL;
	char *textColor = NULL;
	
	if (objc != 2) {
		Tcl_WrongNumArgs (interp, 1, objv, ".path text");
		return TCL_ERROR;
	}

	text = Tcl_GetString (objv[1]);
	
	if (wPtr->text != NULL) {
		free (wPtr->text);
	}

	if (wPtr->tkfont != NULL) {
		Tk_FreeFont (wPtr->tkfont);
	}
	
	/*duplicate because the pointer will become invalid*/
	wPtr->text = safeStrdup (text);
	
	fontName = Tk_GetOption (wPtr->tkwin, "font", "ThemeLabel");
	
	if (fontName == NULL) {
		fontName = "Lucida 12";
	}
	
	wPtr->tkfont = Tk_GetFont (NULL, wPtr->tkwin, fontName);

	if (wPtr->tkfont == NULL) {
		fontName = "Lucida 12";
		wPtr->tkfont = Tk_GetFont (NULL, wPtr->tkwin, fontName);
	}
	

	if (wPtr->tkfont == NULL) {
		Tcl_SetResult (interp, "unable to get font", TCL_STATIC);
		return TCL_ERROR;
	}

	Tk_GetFontMetrics (wPtr->tkfont, &fm);
	
	wPtr->numChars = Tk_MeasureChars (wPtr->tkfont, wPtr->text, strlen (wPtr->text), -1, 0, &length);
	wPtr->textPixWidth = length;
	wPtr->textPixHeight = fm.linespace + 2;
	wPtr->textPixY = fm.ascent;
	wPtr->textGC = XCreateGC (wPtr->dis, wPtr->xWin, 0, 0);
	
	XSetFont (wPtr->dis, wPtr->textGC, Tk_FontId (wPtr->tkfont));

	textColor = Tk_GetOption (wPtr->tkwin, "foreground", "ThemeLabel");

	if (textColor == NULL) {
		textColor = "black";
	}

	if (wPtr->fgCol.pixel != 0) {
		XFreeColors (wPtr->dis, wPtr->colormap, &wPtr->fgCol.pixel, 1, 0L);
	}
	
	XParseColor (wPtr->dis, wPtr->colormap, textColor, &wPtr->fgCol);
	XAllocColor (wPtr->dis, wPtr->colormap, &wPtr->fgCol);
	XSetForeground (wPtr->dis, wPtr->textGC, wPtr->fgCol.pixel);
	
	Tk_GeometryRequest (wPtr->tkwin, wPtr->textPixWidth, wPtr->textPixHeight);
	
	return TCL_OK;
}

int ThemeLabelCmd OBJ_CMD_ARGS {
	char *path = NULL;
	Tk_Window tkParent = Tk_MainWindow (interp);
	Tk_Window tkwin;
	ThemeLabel *wPtr;
	int i;
	char *theText = NULL;
	char *varName = NULL;
	int noBGPixmap = 0;
	
	if (objc < 2) {
		Tcl_WrongNumArgs (interp, 1, objv, ".path");
		return TCL_ERROR;
	}
	
	path = Tcl_GetString (objv[1]);
	
	for (i = 2; i < objc; ++i) {
		char *arg = Tcl_GetString (objv[i]);
		if (strncmp (arg, "-textvariable", 12) == 0) {
			++i;
			if (i >= objc) {
				Tcl_SetResult (interp, "no value for -textvariable", TCL_STATIC);
				return TCL_ERROR;
			}
			
			varName = safeStrdup (Tcl_GetString (objv[i]));
		} else if (strncmp (arg, "-text", 5) == 0) {
			++i;
			if (i >= objc) {
				Tcl_SetResult (interp, "no value for -text", TCL_STATIC);
				return TCL_ERROR;
			}
			
			theText = safeStrdup (Tcl_GetString (objv[i]));
		} else if (strncmp (arg, "-nobgpixmap", 9) == 0) {
			++i;
			if (i >= objc) {
				Tcl_SetResult (interp, "no value for -nobgpixmap", TCL_STATIC);
				return TCL_ERROR;
			}
		
			if (Tcl_GetBooleanFromObj (interp, objv[i], &noBGPixmap) != TCL_OK) {
				return TCL_ERROR;
			}
		} else {
			Tcl_SetResult (interp, "invalid argument", TCL_STATIC);
			return TCL_ERROR;
		}
	}
	
	tkwin = Tk_CreateWindowFromPath (interp, tkParent, path, (char *) NULL);
	if (tkwin == NULL) {
		return TCL_ERROR;
	}
	
	Tk_MakeWindowExist (tkwin);
	Tk_SetClass (tkwin, "ThemeLabel");

	wPtr = (ThemeLabel *) ckalloc (sizeof (ThemeLabel));
	wPtr->tkwin = tkwin;
	wPtr->dis = Tk_Display (tkwin);
	wPtr->path = safeStrdup (path);
	wPtr->text = NULL;
	wPtr->xWin = Tk_WindowId (tkwin);
	wPtr->screen = DefaultScreen (wPtr->dis);
	wPtr->colormap = DefaultColormap (wPtr->dis, wPtr->screen);
	wPtr->depth = DefaultDepth (wPtr->dis, wPtr->screen);
	
	wPtr->tkfont = NULL;
	wPtr->numChars = 0;
	wPtr->textPixWidth = 1;
	wPtr->textPixHeight = 1;
	wPtr->textPixY = 1;
	wPtr->textGC = XCreateGC (wPtr->dis, wPtr->xWin, 0, 0);
	wPtr->fgCol.pixel = 0;
	wPtr->noBGPixmap = noBGPixmap;
	wPtr->varName = varName;
	
	Tcl_CreateObjCommand (interp, path, ThemeLabelInstanceCmd, (ClientData) wPtr, (Tcl_CmdDeleteProc *) NULL);
	
	Tk_CreateEventHandler (tkwin, ExposureMask | StructureNotifyMask, 
		ThemeLabelEvent, (ClientData) wPtr
	);

	Tk_GeometryRequest (tkwin, 1, 1);

	if (theText != NULL) {
		Tcl_Obj *arObj[2];

		arObj[0] = objv[1];
		arObj[1] = Tcl_NewStringObj (theText, strlen (theText));

		if (ThemeLabelInstanceCmd ((ClientData) wPtr, interp, 2, arObj) != TCL_OK) {
			Tcl_DecrRefCount (arObj[1]);
			free (theText);
			return TCL_ERROR;
		}
		Tcl_DecrRefCount (arObj[1]);
		free (theText);
	}

	if (varName != NULL) {
		if (Tcl_TraceVar (interp, varName, TCL_TRACE_WRITES | TCL_GLOBAL_ONLY, \
			ThemeLabelTraceWrite, (ClientData) wPtr) != TCL_OK) {
			return TCL_ERROR;
		}
	
		ThemeLabelTraceWrite (wPtr, interp, varName, NULL, 0);
	}
	
	Tcl_SetResult (interp, path, TCL_VOLATILE);

	return TCL_OK;
}
